<?php
// +---------------------------------------------------------------------------+
// | This file is part of the KnowledgebasePublisher package                   |
// | KnowledgebasePublisher - web based knowledgebase publisher tool           |
// |                                                                           |
// | Author:  Evgeny Leontev <eleontev@gmail.com>                              |
// | Copyright (c) 2005 Evgeny Leontev                                         |
// |                                                                           |
// | For the full copyright and license information, please view the LICENSE   |
// | file that was distributed with this source code.                          |
// +---------------------------------------------------------------------------+

require_once 'eleontev/Util/TreeHelper.php';
require_once 'eleontev/SQL/TableSortOrder.php';


class KBCategoryModel extends AppModel
{
	var $tbl_pref_custom = 'kb_';
	var $tables = array('table'=>'category', 'category', 'entry', 'entry_to_category');
	var $custom_tables = array('data_to_user_rule', 'data_to_user_value', 'user');
	
	
	function getRecordsSql() { 
		$sql = "SELECT c.*, 
		COUNT(ec.entry_id) AS num_entries 
		FROM {$this->tbl->category} c
		LEFT JOIN {$this->tbl->entry_to_category} ec ON c.id = ec.category_id
		WHERE {$this->sql_params}
		GROUP BY c.id
		{$this->sql_params_order}";

		return $sql;
	}		
	
	 
	function &getRecords() {
		$data = array();

		$sql = $this->getRecordsSql();
		$result =& $this->db->Execute($sql) or die(db_error($sql));
		while($row = $result->FetchRow()){
			$data[$row['id']] = $row;
		}

		return $data;
	}
	
	
	// get helper array in format id=>level
	function &getTreeHelperArray($arr, $parent_id = 0) {
		
		if(!$arr) { return array(); }
		
		$tree = &new TreeHelper();
		foreach($arr as $k => $row) {
			$tree->setTreeItem($row['id'], $row['parent_id']);
		}
		
		return $tree->getTreeHelper($parent_id);
	}	
	
	
	
	// FOR FORM SELECT // ---------------------
	
	// just for select
	function &getSelectRecords($sort = false) { 
	
		$sort = ($sort) ? $sort : 'sort_order';
	
		$sql = "SELECT id as id1, id, parent_id, name, sort_order 
		FROM {$this->tbl->category} c
		ORDER BY {$sort}";
		$result =& $this->db->Execute($sql) or die(db_error($sql));
		return $result->GetAssoc();
	}
	
	
	// to generate form select range $arr from getSelectRecords
	function &getSelectRange($arr, $parent_id = 0, $pref = '-- ') {
		
		if(!$arr) { return array(); }
		
		$tree_helper = &$this->getTreeHelperArray($arr, $parent_id);
		foreach($tree_helper as $id => $level) {
	
			$p = ($level == 0) ? '' : str_repeat($pref, $level);
			$data[$id] = $p . $arr[$id]['name'];
		}
		
		return $data;
	}
	
	
	// generate arrays for js (DynamicOptionList) $arr from getSelectRecords
	function &getSortJsArray($arr) {
		
		$values = array();
		foreach($arr as $v) {
			$str =  "'%s: %s', '%s'";
			$values[$v['parent_id']][] = sprintf($str, 'AFTER', addslashes($v['name']), $v['sort_order']);
		}
		
		
		$sort = new TableSortOrder(); $str = '';
		unset($sort->extra_range['sort_default']);
		foreach($sort->extra_range as $k => $v) {
			$str .= sprintf("'%s', '%s', ", $v, $k);
		}
		
		foreach($values as $k => $v) {
			$values[$k] = "'" . $k . "'," . $str . implode(',', $v);
		}
		
		return $values;
	}
	
	
	
	// CATEGORY TO USER
	
	function getAdminUserById($record_id) {
			return $this->_getUserDataById($record_id, 
		                                      'kb_category_to_admin', 
		                                      's_user u', 
											   array('u.id', 'CONCAT(u.first_name, " ", u.last_name)'));
	}
	
	
	function getRuleId($rule) {
		$sql = "SELECT id FROM {$this->tbl->data_to_user_rule} d
		WHERE d.title = '{$rule}'";
		$result =& $this->db->Execute($sql) or die(db_error($sql));
		return $result->Fields('id');
	}
	
	
	function saveAdminUserToCategory($obj, $record_id) {
		$this->_saveUserDataToCategory($obj->getAdminUser(), $record_id, 'kb_category_to_admin');
	}
	
	
	function deleteAdminUserToCategory($record_id) {
		$this->_deleteUserDataToCategory($record_id, 'kb_category_to_admin');
	}
	
	// used when wrong submit to generate list 
	function getAdminUserByIds($ids) {
		$sql = "SELECT u.id, CONCAT(u.first_name, ' ', u.last_name) 
		FROM {$this->tbl->user} u
		WHERE u.id IN ($ids)";
			
		$result =& $this->db->Execute($sql) or die(db_error($sql));
		return $result->GetAssoc();	
	}	
	
	function &_getUserDataById($record_id, $rule, $table, $fields) {
		
		list($user_id, $user_value) = $fields;
		$fields = implode(', ', $fields);
		
		$sql = "
		SELECT 
			{$fields}
		FROM 
			{$this->tbl->data_to_user_rule} d,
			{$this->tbl->data_to_user_value} dv,
			{$table}
		WHERE 1
			AND d.title = '{$rule}'
			AND d.id = dv.rule_id
			AND dv.data_value IN ($record_id)
			AND dv.user_value = {$user_id}";
		
		$result =& $this->db->Execute($sql) or die(db_error($sql));
		return $result->GetAssoc();
	}		
	
	function _saveUserDataToCategory($data, $record_id, $rule) {
		
		if(!$data) { return; }
		
		require_once 'eleontev/SQL/MultiInsert.php';
		
		$rule_id = $this->getRuleId($rule);
		foreach($data as $k => $v) {
			$_data[] = array($v);
		}
		
		$ins = new MultiInsert;
		$ins->setFields('user_value', array('rule_id', 'data_value'));
		$ins->setValues($_data, array($rule_id, $record_id));
		$sql = $ins->getSql($this->tbl->data_to_user_value);
		
		//echo "<pre>"; print_r($sql); echo "</pre>";
		//exit;
		
		return $this->db->Execute($sql) or die(db_error($sql));
	}
	
	
	function _deleteUserDataToCategory($record_id, $rule) {
		
		$rule_id = $this->getRuleId($rule);
		$sql = "DELETE FROM {$this->tbl->data_to_user_value} 
		WHERE rule_id = '{$rule_id}' AND data_value = '{$record_id}'";
		
		return $this->db->Execute($sql) or die(db_error($sql));
	}
		
	
	// ACTIONS // ---------------------
	
	//  return sort val and dod nessesary updates in sort_order field
	function sortManipulation(&$obj) {
		
		$parent_id = $obj->get('parent_id');
		
		$sort = new TableSortOrder();
		$sort->table = $this->tbl->category;
		$sort->name_field = 'name';
		$sort->sort_field = 'sort_order';
		$sort->setMoreSql("parent_id = '{$parent_id}'",
		                  "parent_id = '{$parent_id}'");
		
		$val = $sort->getDoSort($obj->get('sort_order'), $obj->get('id'));
		$obj->set('sort_order', $val);
	}
	
	
	function save(&$obj, $action) {
		
		$this->sortManipulation($obj);
		
		if($action == 'insert') {
			$id = $this->add($obj);
			
			if($this->isExtra('notify_admin_category')) {
				$this->saveAdminUserToCategory($obj, $id);
			}
			
			
		} else {
			$id = $obj->get('id');
			$this->update($obj, $id);
			
			if($this->isExtra('notify_admin_category')) {
				$this->deleteAdminUserToCategory($id);
				$this->saveAdminUserToCategory($obj, $id);
			}
		}
		
		return true;
	}	
	
	
	
	// DELETE RELATED // ---------------------
		
	// return how much entries for current if some product assigned to category 
	function isCategoryInUse($record_id) {
		$sql = "SELECT COUNT(*) AS num FROM {$this->tbl->entry_to_category} 
		WHERE category_id = ?";
		$result =& $this->db->Execute($sql, $record_id) or die(db_error($sql));
		return $result->Fields('num');
	}
	
	
	// delete category
	function deleteCategory($record_id) {
		$sql = "DELETE FROM {$this->tbl->category} WHERE id IN (?)";
		return $this->db->Execute($sql, $record_id) or die(db_error($sql));
	}
	
	
	// change parent when delete a category
	function setCategoryToParent($record_id, $parent_id) {
		$sql = "UPDATE {$this->tbl->category} 
		SET parent_id = '$parent_id'
		WHERE parent_id = '$record_id'";
		return $this->db->Execute($sql) or die(db_error($sql));
	}
	
	
	// change product to parent category when delete a category
	function setCategoryEntryToParent($record_id, $parent_id) {
		$sql = "UPDATE {$this->tbl->entry_to_category} 
		SET category_id = '$parent_id'
		WHERE category_id = '$record_id'";
		return $this->db->Execute($sql) or die(db_error($sql));
	}
	
	
	// change sort_order for all categories with the same parent
	function setCategoryOrder($record_id, $parent_id) {
		
		$sql = "SELECT sort_order FROM {$this->tbl->category} WHERE id = '$record_id'";
		$result = &$this->db->Execute($sql) or die(db_error($sql));
		$order = $result->Fields('sort_order');
		
		$sql = "UPDATE {$this->tbl->category} 
		SET sort_order = (sort_order-1)
		WHERE parent_id = '$parent_id'
		AND sort_order > $order";
		
		return $this->db->Execute($sql) or die(db_error($sql));
	}	
	
	
	function delete($id, $parent_id) {
		
		$ret = 'notdeleteable_category';
		$in_use = $this->isCategoryInUse($id);
		
		// top level category is in use, notdeleteable
		if($parent_id == 0 && $in_use) {
			$ret = 'notdeleteable_category';
		
		// top level category but not in use
		} elseif($parent_id == 0 && !$in_use) {
			
			//DBUtil::begin();
			$this->setCategoryOrder($id, $parent_id);			
			$this->deleteCategory($id);
			$this->setCategoryToParent($id, $parent_id);
			//DBUtil::commit(); 
			
			$ret = 'success';
			
		// child category, all values assigned to parent
		} else {
			
			//DBUtil::begin();
			$this->setCategoryOrder($id, $parent_id);
			$this->deleteCategory($id);
			$this->setCategoryToParent($id, $parent_id);
			$this->setCategoryEntryToParent($id, $parent_id);
			//DBUtil::commit();
			
			$ret = 'success';
		}
		
		return $ret;
	}
}
?>