<?php
// +---------------------------------------------------------------------------+
// | This file is part of the KnowledgebasePublisher package                   |
// | KnowledgebasePublisher - web based knowledgebase publisher tool           |
// |                                                                           |
// | Author:  Evgeny Leontev <eleontev@gmail.com>                              |
// | Copyright (c) 2005 Evgeny Leontev                                         |
// |                                                                           |
// | For the full copyright and license information, please view the LICENSE   |
// | file that was distributed with this source code.                          |
// +---------------------------------------------------------------------------+

require_once APP_MODULE_DIR . 'knowledgebase/category/inc/KBCategoryModel.php';

/*
attachment type
1 - attachment
2 - inline file (link in document)
3 - exist as attachment and as inline
*/


class KBEntryModel extends AppModel
{
	var $tbl_pref_custom = 'kb_';
	var $tables = array('table'=>'entry', 'entry', 'category', 'entry_to_category', 
	                    'comment', 'rating', 'attachment_to_entry', 'entry_to_related', 'entry_by_user');
	
	var $custom_tables = array('file_entry');
	
	
	var $attachment_dir;
	var $attachment_path;
	var $use_attachment = false;
	
	
	function getById($record_id) {
		$sql = "SELECT e.*, UNIX_TIMESTAMP(e.date_posted) AS ts
		FROM {$this->tbl->entry} e
		WHERE e.id = ?";
		
		//echo "<pre>"; print_r($sql); echo "</pre>";
		$result =& $this->db->Execute($sql, $record_id) or die(db_error($sql));
		return $result->FetchRow();
	}

	
	function getRecordsSql() {
		
		if($this->isExtra('file')) {
			$att_select = 'COUNT(DISTINCT a.attachment_id)';
			$att_from = "LEFT JOIN {$this->tbl->attachment_to_entry} a ON e.id = a.entry_id";
		} else {
			$att_select = '0';
			$att_from = '';		
		}
		
		
		
		$sql = "
		SELECT 
			e.*,
			IF(e.body != '', e.active, 'not') AS active,
			CONCAT(SUBSTRING_INDEX(e.body,' ',20), '...') AS body, 
			cat.id AS category_id,
			cat.name AS category_name,
			#cat.private AS category_private,
			cat.active AS category_active,
			r.votes AS votes,
			(r.rate/r.votes) AS rating,
			COUNT(DISTINCT c.id) AS comment_num,
			$att_select AS attachment_num,
			UNIX_TIMESTAMP(e.date_posted) AS ts
			
		FROM 
			({$this->tbl->entry} e,
			{$this->tbl->category} cat,
			{$this->tbl->entry_to_category} e_to_cat)
		
		LEFT JOIN {$this->tbl->comment} c ON e.id = c.entry_id
		LEFT JOIN {$this->tbl->rating} r ON e.id = r.entry_id	
		$att_from
		
		WHERE 1
			AND e.id = e_to_cat.entry_id
			AND cat.id = e_to_cat.category_id
			AND {$this->sql_params}
		GROUP BY e.id
		{$this->sql_params_order}";
		
		//echo "<pre>"; print_r($sql); echo "</pre>";
		return $sql;
	}		
	
	
	// for page by page
	function getCountRecordsSql() {
		$sql = "SELECT COUNT(DISTINCT(e.id)) 
		FROM 
			{$this->tbl->entry} e,
			{$this->tbl->category} cat,
			{$this->tbl->entry_to_category} e_to_cat		
		WHERE e.id = e_to_cat.entry_id
		AND cat.id = e_to_cat.category_id
		AND {$this->sql_params}";
		return $sql;
	}	
	
	
	// USERS QUESTIONS // ---------------------------	
	
	function getUserEntryById($id) {
		$sql = "SELECT * FROM {$this->tbl->entry_by_user} WHERE id = %d";
		$result =& $this->db->Execute(sprintf($sql, $id)) or die(db_error($sql));
		return $result->FetchRow();
	}
	
	function setUserEntryPlaced($record_id) {
		$sql = "UPDATE {$this->tbl->entry_by_user} SET placed = 1 WHERE id = '{$record_id}'";
		$this->db->Execute($sql) or die(db_error($sql));
	}	
	
	
	// CATEGORIES // ---------------------------
	
	// return categories array for entry
	function &getCategoryById($record_id) {
		$sql = "SELECT category_id AS id, category_id FROM {$this->tbl->entry_to_category} 
		WHERE entry_id IN ($record_id)";
		$result =& $this->db->Execute($sql) or die(db_error($sql));
	
		return $result->GetAssoc();
	}
	
	
	// get array with categories names to use in list records
	function &getCategoryByIds($record_id) {
		
		$sql = "
		SELECT 
			ec.entry_id, 
			c.name 
		FROM 
			{$this->tbl->category} c, 
			{$this->tbl->entry_to_category} ec
		WHERE 1
			AND ec.entry_id IN ($record_id) 
			AND c.id = ec.category_id";
		$result =& $this->db->Execute($sql) or die(db_error($sql));
		while($row = $result->FetchRow()){
			$data[$row['entry_id']][] = $row['name'];
		}
		
		return $data;
	}
	
	
	function getCategoryRecords() {
		$this->cat = new KBCategoryModel();
		return $this->cat->getSelectRecords();
	}
	
	
	// range for form select
	function &getCategorySelectRange($rows = false) {
		if(!$rows) {
			$rows = $this->getCategoryRecords();
		}
		return $this->cat->getSelectRange($rows);
	}
	
	
	function getChilds($rows, $id) {
		
		$tree = &new TreeHelper();
		foreach($rows as $k => $row) {
			$tree->setTreeItem($row['id'], $row['parent_id']);
		}
		
		return $tree->getChildsById($id);
	}
	
	
	// ATTACHMENT // -------------------------
	
	// get array with attachments filenames
	function &getAttachmentById($record_id, $attachment_type = '1,3') {
		
		$sql = "
		SELECT 
			f.id, 
			f.filename 
		FROM 
			{$this->tbl->file_entry} f, 
			{$this->tbl->attachment_to_entry} ae
		WHERE 1
			AND ae.entry_id IN ($record_id) 
			AND ae.attachment_id = f.id
			AND ae.attachment_type IN ($attachment_type)";
			
		$result =& $this->db->Execute($sql) or die(db_error($sql));
		return $result->GetAssoc();
	}
	
	
	function getFilesByIds($ids) {
		$sql = "SELECT f.id, f.filename 
		FROM {$this->tbl->file_entry} f
		WHERE f.id IN ($ids)";
		
		$result =& $this->db->Execute($sql) or die(db_error($sql));
		return $result->GetAssoc();
	}
	
	
	function getInlineFilesIds($str) {
	
		if(strpos($str, '[link:file') === false) {
			return array();
		}
		
		$search = "/\[link:file\|(\d+)\]/";
		//$search = "/\[link:(\w+)\|(\d+)\|?(\d{0,})\]/e";
		
		preg_match_all($search, $str, $match);
		return $match[1];
	}
	
	
	
	// RELATED // ---------------------	
	
	// get array with related titles
	function &getRelatedById($record_id) {
		
		$sql = "
		SELECT 
			e.id, 
			e.title 
		FROM 
			{$this->tbl->entry} e, 
			{$this->tbl->entry_to_related} er
		WHERE 1
			AND er.entry_id IN ($record_id) 
			AND er.related_entry_id = e.id";
			
		$result =& $this->db->Execute($sql) or die(db_error($sql));
		return $result->GetAssoc();
	}
	
	
	function getRelatedByIds($ids) {
		$sql = "SELECT e.id, e.title 
		FROM {$this->tbl->entry} e
		WHERE f.id IN ($ids)";
			
		$result =& $this->db->Execute($sql) or die(db_error($sql));
		return $result->GetAssoc();
		
	}
	
	
	// ACTIONS // ---------------------
	
	function addEntry(&$obj) {
		return $this->add($obj);
	}	
	
	
	function updateEntry(&$obj) {		
		$this->update($obj);
	}
	
	
	function saveEntryToCategory(&$obj, $record_id) {
		
		require_once 'eleontev/SQL/MultiInsert.php';
		
		$cat = $obj->getCategory();
		$cat = (is_array($cat)) ? $cat : array($cat);
		foreach($cat as $k => $v) {
			$categories[] = array($v);
		}
		
		$sql = MultiInsert::get("INSERT {$this->tbl->entry_to_category} (category_id, entry_id) 
		                         VALUES ?", $categories, array($record_id));
		
		return $this->db->Execute($sql) or die(db_error($sql));
	}
	
	
	function saveAttachmentToEntry(&$obj, $record_id) {
		
		$inline_ids = $this->getInlineFilesIds($obj->get('body'));
		if(!$obj->getAttachment() && !$inline_ids) { return; }
		
		require_once 'eleontev/SQL/MultiInsert.php';
		
		$cat = $obj->getAttachment();
		foreach($cat as $k => $v) {
			$attachment[$v] = array(1, $v);
		}
		
		foreach($inline_ids as $k => $v) {
			$type = (isset($attachment[$v])) ? 3 : 2; // attachment type
			$attachment[$v] = array($type, $v);
		}		
		
		
		$sql = MultiInsert::get("INSERT {$this->tbl->attachment_to_entry} (attachment_type, attachment_id, entry_id) 
		                        VALUES ?", $attachment, array($record_id));
		
		//echo "<pre>"; print_r($inline_ids); echo "</pre>";
		//echo "<pre>"; print_r($sql); echo "</pre>";
		//exit;
		
		return $this->db->Execute($sql) or die(db_error($sql));
	}

	
	function saveRelatedToEntry(&$obj, $record_id) {
		
		if(!$obj->getRelated()) { return; }
		
		require_once 'eleontev/SQL/MultiInsert.php';
		
		$cat = $obj->getRelated();
		$cat = (is_array($cat)) ? $cat : array($cat);
		foreach($cat as $k => $v) {
			$entry[] = array($v);
		}
			
		$sql = MultiInsert::get("INSERT IGNORE {$this->tbl->entry_to_related} (related_entry_id, entry_id) 
		                        VALUES ?", $entry, array($record_id));
		
		return $this->db->Execute($sql) or die(db_error($sql));
	}
	
	
	function save(&$obj) {
		
		$this->sortManipulation($obj);		
		
		// insert
		if(!$obj->get('id')) {
			
			$id = $this->addEntry($obj);
			
			$this->saveEntryToCategory($obj, $id);
			//$this->saveRelatedToEntry($obj, $id);
			
			if($this->isExtra('file')) {
				$this->saveAttachmentToEntry($obj, $id);
			}
		
		// update
		} else {
			
			$id = $obj->get('id');
			
			$this->updateEntry($obj);
			$this->deleteEntryToCategory($id);
			$this->saveEntryToCategory($obj, $id);
			
			//$this->deleteRelatedToEntry($id);
			//$this->saveRelatedToEntry($obj, $id);	
			
			if($this->isExtra('file')) {
				$this->deleteAttachmentToEntry($id);
				$this->saveAttachmentToEntry($obj, $id);
			}
		}
		
		return $id;
	}
	
	
	//  return sort val and do nessesary updates in sort_order field
	function sortManipulation(&$obj) {
		
		$sort = new TableSortOrder();
		$sort->table = $this->tbl->entry;
		$sort->name_field = 'name';
		$sort->sort_field = 'sort_order';
		$sort->more_update_sql = ", date_updated = date_updated";
		
		// -1 in $obj->get('sort_order') just a trick need to see class code more...
		$val = $sort->getDoSort($obj->get('sort_order')-1, $obj->get('id'));
		$obj->set('sort_order', $val);
	}
	
	
	// PRIV // ------------------------------	
	
	// if check priv is different for model so reassign 	
	function checkPriv(&$priv, $action, $record_id) {
		
		$priv->setCustomAction('answer', 'insert');
		$priv->setCustomAction('file', 'update');
		//$priv->setCustomAction('delete_file', 'delete');		
		
		$priv->setOwnParam(sprintf("author_id=%d", $priv->user_id));
		
		$sql = "SELECT 1 FROM {$this->tbl->table} 
		WHERE id = '{$record_id}'
		AND author_id = {$priv->user_id}";
		$priv->setOwnSql($sql);
		$priv->check($action);
		
		// set sql to select own records
		$this->setSqlParams('AND ' . $priv->getOwnParam());
	}
	
	
	// DELETE RELATED // --------------------- 
	
	function deleteEntries($record_id) {
		$sql = "DELETE FROM {$this->tbl->entry} WHERE id IN ({$record_id})";
		return $this->db->Execute($sql) or die(db_error($sql));
	}
	
	function deleteRating($record_id) {
		$sql = "DELETE FROM {$this->tbl->rating} WHERE entry_id IN ({$record_id})";
		return $this->db->Execute($sql) or die(db_error($sql));
	}
	
	function deleteComments($record_id) {
		$sql = "DELETE FROM {$this->tbl->comment} WHERE entry_id IN ({$record_id})";
		return $this->db->Execute($sql) or die(db_error($sql));
	}
	
	function deleteEntryToCategory($record_id) {
		$sql = "DELETE FROM {$this->tbl->entry_to_category} WHERE entry_id IN ({$record_id})";
		return $this->db->Execute($sql) or die(db_error($sql));
	}
	
	// delete by entry_id from related if any
	// so we do not have any related to this entry
	function deleteRelatedToEntry($record_id) {
		$sql = "DELETE FROM {$this->tbl->entry_to_related} WHERE entry_id IN ({$record_id})";
		return $this->db->Execute($sql) or die(db_error($sql));
	}
	
	// delete by related_entry_id from related if any
	// so we do not have any entry that related to this entry	
	function deleteEntryToRelated($record_id) {
		$sql = "DELETE FROM {$this->tbl->entry_to_related} WHERE related_entry_id IN ({$record_id})";
		return $this->db->Execute($sql) or die(db_error($sql));
	}	
	
	function deleteAttachmentToEntry($record_id) {
		$sql = "DELETE FROM {$this->tbl->attachment_to_entry} WHERE entry_id IN ({$record_id})";
		return $this->db->Execute($sql) or die(db_error($sql));
	}
	
	
	function delete($record_id) {
		
		// convert to string 1,2,3... to use in IN()
		$record_id = $this->idToString($record_id);
		
		$this->deleteEntries($record_id);
		$this->deleteRating($record_id);
		$this->deleteComments($record_id);
		$this->deleteEntryToCategory($record_id);
		//$this->deleteRelatedToEntry($record_id);
		//$this->deleteEntryToRelated($record_id);
		
		if($this->isExtra('file')) {
			$this->deleteAttachmentToEntry($record_id);
		}
	}
}
?>