<?php
// +---------------------------------------------------------------------------+
// | This file is part of the KnowledgebasePublisher package                   |
// | KnowledgebasePublisher - web based knowledgebase publisher tool           |
// |                                                                           |
// | Author:  Evgeny Leontev <eleontev@gmail.com>                              |
// | Copyright (c) 2005 Evgeny Leontev                                         |
// |                                                                           |
// | For the full copyright and license information, please view the LICENSE   |
// | file that was distributed with this source code.                          |
// +---------------------------------------------------------------------------+


class KBClientView_default extends KBClientView_common
{
	
	var $num_category_td = 3;
		

	function &execute(&$manager) {
		
		if(!$this->category_id) {
			$rows = &$this->stripVars($manager->getCategoryList($this->top_parent_id));
			$title = $this->msg['category_title_msg'];
			$meta_title = '';
			
		} else {
			$rows = &$this->stripVars($manager->getCategoryList($this->category_id));
			$title =  $this->msg['subcategory_title_msg'];
			$meta_title = $manager->categories[$this->category_id]['name'];
		}
		

		$this->meta_title = $meta_title;

		$data = array();
		$data[] = &$this->getTop($manager);
		$data[] = &$this->getCategoryList($rows, $title, $manager);
		$data[] = &$this->getEntryList($manager);	
		$data = implode('', $data);
		
		return $data;
	}
	

	function getEntriesNumFormated($num_entry_format, $num_entries, $num_categories = false) {
		
		$str[0] = "";
		$str[1] = "(%s)";
		$str[2] = (!$num_categories) ? $str[1] : "(%s / %s)";
		
		return sprintf($str[$num_entry_format], $num_entries, $num_categories);
	}
	
	
	function &getCategoryList($rows, $title, $manager) {
		
		if(!$rows || !$this->display_categories) { $a = ''; return $a; }
		
		$num = count($rows);
		$rows = array_chunk($rows, $this->num_category_td);
		
		// td width
		if($num < $this->num_category_td) { $td_width = round(100/$num); } 
		else                              {	$td_width = round(100/$this->num_category_td); }
		
		
		$tpl = &new tplTemplatez($this->template_dir . 'category_list.html');
		
		foreach($rows as $k => $v) {
			foreach($v as $k1 => $v1) {
				
				$v1['td_width']	= $td_width;
				$v1['num_entries'] = $this->getEntriesNumFormated($manager->getSetting('show_num_entries'), $v1['num_entries'], @$v1['num_subcategories']);
				$v1['category_link'] = $this->_getLink('', $v1['id'], false);
				$v1['item_img'] = $this->_getItemImg($manager->is_registered, $v1['private'], true);
				
				$tpl->tplParse($v1, 'row_tr/row_td'); // parse nested
			}
		
			// do it nested
			$tpl->tplSetNested('row_tr/row_td');
			$tpl->tplParse('', 'row_tr');	
		}
		
		$tpl->tplAssign('title_msg', $title);
		$tpl->tplAssign($this->msg);
		$tpl->tplParse();
		
		return $tpl->tplPrint(1);
	}
	
	
	
	// parse data with articles
	function &_parseArticleList(&$manager, &$rows, $title, $by_page = '') {
		
		if(!$rows) { $a = ''; return $a; }
		
		$tpl = &new tplTemplatez($this->template_dir . 'article_list.html');
		
		foreach($rows as $k => $row) {
			
			$tpl->tplAssign('item_img', $this->_getItemImg($manager->is_registered, $row['category_private']));
			$tpl->tplAssign('updated_date', $this->getFormatedDate($row['ts_updated']));
			$tpl->tplAssign('entry_link', $this->_getLink('entry', $row['category_id'], $row['id']));

			$row['body'] = DocumentParser::getSummary($row['body']); 
			
			if($manager->getSetting('show_hits')) {
				$tpl->tplSetNeeded('row/show_hits');
			}
			
			if($this->isRatingable($manager, $row['ratingable'])) {
				$tpl->tplAssign('rating', $this->_getRating($row['rating']));
			} else {
				$tpl->tplAssign('rating', '');
			}	
			
			if($this->isCommentable($manager, $row['commentable'])) {
				$tpl->tplSetNeeded('row/show_comments');
			}			
			
			$tpl->tplParse(array_merge($row, $this->msg), 'row');
		} 
		
		//show more link if no categories
		if(!$this->display_categories) {
			$tpl->tplAssign('more_link', $this->_getLink($row['category_id']));
			$tpl->tplAssign('_more_msg', 'more');
		}
		
		$tpl->tplAssign('page_by_page', $by_page);
		$tpl->tplAssign('entry_title_msg', $title);
		$tpl->tplParse();
		
		return $tpl->tplPrint(1);
	}
	
	
	function &getEntryList(&$manager) {
		
		$manager->setSqlParams('AND ' . $manager->getPrivateSql(false));
		
		// category articles
		if($this->category_id) {
			
			require_once 'eleontev/PageByPage.php';
	
			$manager->setSqlParams("AND cat.id = {$this->category_id}");
			$bp = $this->pageByPage($manager->getSetting('num_entries_per_page'), $manager->getEntryCountSql());
			
			$manager->setSqlParamsOrder('ORDER BY sort_order');			
			$rows = &$manager->getEntryList($bp->limit, $bp->offset);
			return $this->_parseArticleList($manager, $this->stripVars($rows), $this->msg['entry_title_msg'], $bp->shortNavigate());
		
		
		// top articles
		} else {
			
			$most_num   = $manager->getSetting('num_most_viewed_entries');
			$recent_num = $manager->getSetting('num_recently_posted_entries');
			
			if($most_num && $recent_num) {
				$tpl = &new tplTemplatez($this->template_dir . 'top_article_format.html');
				
				$data = array();
				$data['data1'] = &$this->_getMostViewed($manager, $most_num);
				$data['data2'] = &$this->_getRecentlyPosted($manager, $recent_num);
				
				$tpl->tplParse($data);			
				return $tpl->tplPrint(1);				
			
			} elseif($most_num) {
				return $this->_getMostViewed($manager, $most_num);
				
			} elseif($recent_num) {
				return $this->_getRecentlyPosted($manager, $recent_num);
			}
		}
	}
	
	
	function &_getMostViewed($manager, $num) {
		$manager->setSqlParamsOrder('ORDER BY hits DESC');
		$rows = &$manager->getEntryList($num, 0);
		return $this->_parseArticleList($manager, $this->stripVars($rows), $this->msg['most_viewed_entries_title_msg']);
	}
	
	
	function &_getRecentlyPosted($manager, $num) {
		$manager->setSqlParamsOrder('ORDER BY date_posted DESC');
		$rows = &$manager->getEntryList($num, 0);
		return $this->_parseArticleList($manager, $this->stripVars($rows), $this->msg['recently_posted_entries_title_msg']);
	}	
}
?>